(*----------------------------------------------------------------------------
Author:   Edouard Hibon
Updated on:  November , 1994.
------------------------------------------------------------------------------*)

specification CW [PotsGate,PotsTalk,PotsHang,LinkCw]

:noexit

library Set, Boolean, DecDigit, NaturalNumber, BasicNaturalNumber endlib

type    UsersList is DecDigit, Boolean, BasicNaturalNumber

sorts   List

opns    {}                                      : -> List
	InsertL, RemoveL                        : DecDigit, List -> List
	_IsIn_, _NotIn_                         : DecDigit, List -> Bool
	_Union_, _Ints_, _Minus_                : List, List -> List
	_eq_, _ne_, _Includes_, _IsSubsetOf_    : List, List -> Bool
	Card                                    : List -> Nat
	First                                   : List -> DecDigit

eqns    forall  x, y    : DecDigit,
	        s, t    : List

	ofsort  List

	        x IsIn InsertL(y,s) =>
	InsertL(x, InsertL(y,s))  = InsertL(y,s)                   ;

	RemoveL(x, {}) = {}                                      ;
	RemoveL(x, InsertL(x,s))  = s                             ;
	        x ne y =>
	RemoveL(x, InsertL(y,s))  = InsertL(y, RemoveL(x,s))        ;

	{} Union s              = s                             ;
	InsertL(x,s) Union t     = InsertL(x,s Union t)           ;

	{} Ints s               = {}                            ;
	        x IsIn t =>
	InsertL(x,s) Ints t      = InsertL(x,s Ints t)            ;
	        x NotIn t =>
	InsertL(x,s) Ints t      = s Ints t                      ;

	s Minus {}              = s                             ;
	s Minus InsertL(x, t)    = RemoveL(x,s) Minus t           ;

	ofsort  Bool
	x IsIn {}               = false                         ;
	        x eq y =>
	x IsIn InsertL(y,s)      = true                          ;
	        x ne y =>
	x IsIn InsertL(y,s)      = x IsIn s                      ;

	x NotIn s               = not(x IsIn s)                 ;

	s Includes {}           = true                          ;
	s Includes InsertL(x,t)  = (x IsIn s) and (s Includes t) ;

	s IsSubsetOf t          = t Includes s                  ;

	s eq t                  = (s Includes t) and (t Includes s);

	s ne t                  = not(s eq t)                   ;

	ofsort  DecDigit

	First({})                = 0                         ;
	        x NotIn s =>
	First(InsertL(x,s))       = x                            ;

	ofsort  Nat
	Card ({})                = 0                            ;
	        x NotIn s =>
	Card (InsertL(x,s))       = Succ(Card(s))                      ;
endtype

type    EventSeq is NaturalNumber, Boolean
sorts   EventSeq
opns    0, 1, 2, 3, 4, 5, 6, 7, 8, 9            : -> EventSeq
	_eq_, _ne_, _lt_, _le_, _ge_, _gt_      : EventSeq, EventSeq -> Bool
	NatNum                                  : EventSeq -> Nat

eqns    forall  x, y, z : EventSeq

	ofsort  Nat

	NatNum(0)       = 0                     ;
	NatNum(1)       = Succ(NatNum(0))       ;
	NatNum(2)       = Succ(NatNum(1))       ;
	NatNum(3)       = Succ(NatNum(2))       ;
	NatNum(4)       = Succ(NatNum(3))       ;
	NatNum(5)       = Succ(NatNum(4))       ;
	NatNum(6)       = Succ(NatNum(5))       ;
	NatNum(7)       = Succ(NatNum(6))       ;
	NatNum(8)       = Succ(NatNum(7))       ;
	NatNum(9)       = Succ(NatNum(8))       ;

	ofsort  Bool

	x eq y  = NatNum(x) eq NatNum(y)        ;
	x ne y  = NatNum(x) ne NatNum(y)        ;
	x lt y  = NatNum(x) lt NatNum(y)        ;
	x le y  = NatNum(x) le NatNum(y)        ;
	x ge y  = NatNum(x) ge NatNum(y)        ;
	x ge y  = NatNum(x) ge NatNum(y)        ;

endtype

type DigitPair is DecDigit, Boolean

   sorts DigitPair
   opns
       None    : -> DecDigit
       Pair:DecDigit,DecDigit -> DigitPair
       _eq_: DigitPair, DigitPair -> Bool

   eqns

       forall f1,f2,l1,l2:DecDigit , f3:Nat

       ofsort Bool
	  f1 eq None     = false;
	  None     eq f1 = false;
	  Pair(f1,l1) eq Pair(f2,l2) = (f1 eq f2) and (l1 eq l2);

       ofsort Nat
	  NatNum(None    ) =  0
endtype

type SetOfPairs  is DigitPair, NaturalNumber
     sorts SetOfPairs
     opns
	{}:                                          -> SetOfPairs
	InsertPair:            DigitPair,SetOfPairs  -> SetOfPairs
	RemovePair:            DigitPair,SetOfPairs  -> SetOfPairs
	InsertPairAtTheEnd:    DigitPair,SetOfPairs  -> SetOfPairs
	IsPairIn :             DigitPair,SetOfPairs  -> Bool
	SecondElement:         DecDigit,SetOfPairs   -> DecDigit
	(* given the first element, find the second element *)
	FirstElement:          DecDigit,SetOfPairs   -> DecDigit
	(* given the second element, find the first element *)
	IsPairNotIn:           DigitPair,SetOfPairs  -> Bool
	CardPair:              DecDigit,SetOfPairs   -> Nat
	(* Number of pairs with the same first element Insert the set *)

   eqns forall x,y: DigitPair,
	         s: SetOfPairs,
    f1, f2, s1, s2: DecDigit

   ofsort SetOfPairs
	InsertPairAtTheEnd(x,{})              = InsertPair(x,{});
	InsertPairAtTheEnd(x,InsertPair(x,s)) = InsertPair(x,s) ;
	not(x eq y) =>
	InsertPairAtTheEnd(x,InsertPair(y,s)) =
	                                 InsertPair(y,InsertPairAtTheEnd(x,s));

	RemovePair(x,{}) = {};
	RemovePair(x,InsertPair(x,s)) = s;
	not(x eq y) =>
	RemovePair(x,InsertPair(y,s)) = InsertPair(y,RemovePair(x,s));

   ofsort Bool
	IsPairIn(x,{}) = false;
	IsPairIn(x,InsertPair(y,s)) = (x eq y) or (IsPairIn(x,s));
	IsPairIn(x,RemovePair(x,s)) = false;
	not(x eq y) =>
	IsPairIn(x,RemovePair(y,s)) = IsPairIn(x,s);
	IsPairNotIn(x,s) = not(IsPairIn(x,s));

   ofsort DecDigit
	SecondElement (f1,{}) = None     ;
	SecondElement (f1,InsertPair(Pair(f1,s1),s)) = s1;
	f1 ne f2 =>
	SecondElement (f1,InsertPair(Pair(f2,s2),s)) = SecondElement (f1,s);

	FirstElement (f1,{}) = None     ;     (* not tested *)
	FirstElement (s1,InsertPair(Pair(f1,s1),s)) = f1;
	s1 ne s2 =>
	FirstElement (s1,InsertPair(Pair(f2,s2),s)) = FirstElement (s1,s);

   ofsort  Nat
	CardPair (f1, {})                = 0;
	IsPairNotIn(Pair(f2, s1), s) and (f1 eq f2) =>
	CardPair (f1, InsertPair(Pair(f2, s1), s)) = Succ(CardPair(f1,s));
	IsPairNotIn(Pair(f2, s1), s) and not(f1 eq f2) =>
	CardPair (f1, InsertPair(Pair(f2, s1), s)) = CardPair(f1,s);
endtype

type DecSet0 is Set
     actualizedby DecDigit
     using
     sortnames DecDigit for Element
	       Bool for FBool
endtype

type DecSet is DecSet0
     renamedby
     sortnames  DecSet for Set
endtype

type Signal is Boolean, NaturalNumber
     sorts  Signal
     opns OffHookToCall,
	  OffHookToAnswer,
	  HangsUp,
	  TalksTo,
	  RingsFrom,
	  StopToRingFrom,
	  RingsBackFrom,
	  Dials,
	  GetsTone,
	  GetsBusySignalFrom,
	  GetsCwTone,
	  FlashHook,
	  NotOnLine,
	  OnLine          :-> Signal

	  _IsEq_          : Signal, Signal -> Bool
	  _IsNe_          : Signal, Signal -> Bool
	  Map             : Signal -> Nat

     eqns forall x, y: Signal

     ofsort  Nat
	Map (OffHookToCall)       =  0;
	Map (OffHookToAnswer)     =  Succ(Map (OffHookToCall));
	Map (HangsUp)             =  Succ(Map (OffHookToAnswer));
	Map (TalksTo)             =  Succ(Map (HangsUp));
	Map (RingsFrom)           =  Succ(Map (TalksTo));
	Map (StopToRingFrom)      =  Succ(Map (RingsFrom));
	Map (RingsBackFrom)       =  Succ(Map (StopToRingFrom));
	Map (Dials)               =  Succ(Map (RingsBackFrom));
	Map (GetsTone)            =  Succ(Map (Dials));
	Map (GetsBusySignalFrom)  =  Succ(Map (GetsTone));
	Map (GetsCwTone)          =  Succ(Map (GetsBusySignalFrom));
	Map (FlashHook)           =  Succ(Map (GetsCwTone));
	Map (NotOnLine)           =  Succ(Map (FlashHook));
	Map (OnLine)              =  Succ(Map (NotOnLine));

    ofsort  Bool
	x IsEq y =  Map   (x) eq Map   (y);
	x IsNe y =  Map   (x) ne Map   (y);
endtype


behaviour

(
     Cw [PotsGate,PotsTalk,PotsHang,LinkCw]
)

where

     process Cw [PotsGate,PotsTalk,PotsHang,LinkCw]
     :noexit:=
     (
               InitConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
               ||
               NewGlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
     )
     endproc (* Cw *)

     process InitConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
     :noexit:=
     (
          (
             InitNewConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
             >> InitConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
          )
          |||
          i; InitConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
     )
     endproc (* InitConnection *)


     process InitNewConnection [PotsGate,PotsTalk,PotsHang,LinkCw]
     :exit:=
     (
        PotsGate  ?Caller : DecDigit   !OffHookToCall ;
        (
           PotsGate  !Caller  !GetsTone ;
           (
              PotsGate  !Caller  !Dials  ?Called:DecDigit ;
              (
                  CalledRingsFromCaller [PotsGate,PotsTalk,PotsHang,LinkCw]
                                        (Caller,Called)
               []
                  CalledGetsCwToneFromCaller [PotsGate,PotsTalk,PotsHang,LinkCw]
                                             (Caller,Called)
               []
                  CallerGetsBusySignalFromCalled [PotsGate,PotsHang]
                                                 (Caller,Called)
               []
                  UserHangsUpOnPotsHang [PotsHang] (Caller)
              )
              []  UserHangsUpOnPotsHang [PotsHang] (Caller)
           )
           []  UserHangsUpOnPotsHang [PotsHang] (Caller)
        )
     )
     endproc (* InitNewConnection *)

     process UserHangsUpOnPotsHang [PotsHang] (User:DecDigit)
     :exit:=
     (
        PotsHang  !User  !HangsUp ;
        exit
     )
     endproc (* UserHangsUpOnPotsHang *)

     process CallerGetsBusySignalFromCalled [PotsGate,PotsHang]
                                            (Caller,Called:DecDigit)
     :exit:=
     (
        PotsGate  !Caller  !GetsBusySignalFrom  !Called ;
        UserHangsUpOnPotsHang [PotsHang] (Caller)
     )
     endproc (* CallerGetsBusySignalFromCalled *)

     process CalledRingsFromCaller [PotsGate,PotsTalk,PotsHang,LinkCw]
                                   (Caller,Called:DecDigit)
     :exit:=
     (
        PotsGate  !Called  !RingsFrom  !Caller ;
        (
           PotsGate  !Caller  !RingsBackFrom  !Called ;
           (
		PotsGate  !Called  !OffHookToAnswer  !Caller ;
		CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,LinkCw]
				    (Caller,Called)
            []  UserHangsUpOnPotsHang [PotsHang] (Caller)
           )
          []    UserHangsUpOnPotsHang [PotsHang] (Caller)
        )
     )
     endproc (* CalledRingsFromCaller *)

     process CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,LinkCw]	(* B *)
				 (Caller,Called : DecDigit)
     :exit:=
     (
        PotsConversation [PotsTalk] (Caller,Called)
        [>
        (
           (
              PotsHang  !Caller  !HangsUp ;
              (
                    UserHangsUpOnPotsHang [PotsHang] (Called)
                []
                    LinkCw  !Called  !GetsCwTone ;
                    XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (Called,Caller)
              )
           )
          []
           (
              PotsHang  !Called  !HangsUp ;
              (
                    UserHangsUpOnPotsHang [PotsHang] (Caller)
                []
                    LinkCw  !Caller  !GetsCwTone ;
                    XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (Caller,Called)
              )
           )
          []
           (
              LinkCw  !Caller  !GetsCwTone ;
              XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Caller,Called)
	   )
           []
	   (
              LinkCw  !Called  !GetsCwTone ;
	      XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Called,Caller)
           )
        )
     )
     endproc (* CallerTalksToCalled *)

     process PotsConversation [PotsTalk]  (Caller,Called:DecDigit)
     :noexit:=
     (
               PotsTalk  !Caller  !TalksTo  !Called ;
               PotsConversation [PotsTalk] (Caller,Called)
          []
               PotsTalk  !Called  !TalksTo  !Caller ;
               PotsConversation [PotsTalk] (Caller,Called)
     )
     endproc (* PotsConversation *)

     process CalledGetsCwToneFromCaller [PotsGate,PotsTalk,PotsHang,LinkCw]
					(Caller,Called : DecDigit) 
     :exit:=
     (
        LinkCw  !Called  !GetsCwTone  !Caller ;
        (
           (
              PotsGate  !Caller  !RingsBackFrom  !Called ;
              (
                 (
                    LinkCw  !Called  !FlashHook ;
                    (
			  LinkCw  ?U:DecDigit  !NotOnLine  !Called ;
                    	  CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,
					       LinkCw] (Caller,Called)
		      []  LinkCw  ?U:DecDigit  !OnLine  !Called ;
			  XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] 
				      (Called,Caller)
		    )
                 )
                []
                 (
                    LinkCw  !Called  !HangsUp  !Caller ;
                    XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] 
				      (Called,Caller)
                 )
                []
                 (
                    PotsHang  !Caller  !HangsUp ;
                    XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] 
				      (Called,Caller)
                 )
              )
           )
          []
           (
              PotsHang  !Caller  !HangsUp ;
              XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] 
				(Called,Caller)
           )
          []
           (
              LinkCw  !Called  !HangsUp  !Caller ;
	      (
		 (
		    PotsHang  !Caller  !HangsUp ;
		    XandYHangUpWithYHolding [PotsGate,PotsHang] (Called,Caller)
		 )
		[]
		 (
		    PotsGate  !Called  !RingsFrom  !Caller ;
		    (
			  PotsHang  !Caller  !HangsUp ;
			  XRingsFromYwithYNotOnLine [PotsGate,PotsHang] 
						    (Called,Caller)
		      []  PotsGate  !Caller  !RingsBackFrom  !Called ; 
			  XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,
						   LinkCw] (Called,Caller)
		    )
		 )
	      )
           )
        )
     )
     endproc (* CalledGetsCwToneFromCaller *)

     process XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw](X,Y:DecDigit) (*C*)
     :exit:=
     (
          PotsConversation [PotsTalk]  (X,Y)
          [>
          (
               (
                    PotsHang  !X  !HangsUp ;
                    UserHangsUpOnPotsHang [PotsHang] (Y)
               )
               []
               (
                    PotsHang  !Y  !HangsUp ;
                    XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (X,Y)
               )
               []
               (
                    LinkCw  !X  !FlashHook  !Y ;
                    XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
               )
               []
               (
                    LinkCw  !Y  !GetsCwTone ;
                    XandYGetCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
               )
          )
     )
     endproc (* PotsControlXGetsCwTone *)

     process XGetsCwToneAndYHangsUp [PotsHang,LinkCw] 
				    (X,Y : DecDigit) 			(* D *)
     :exit:=
     (
               UserHangsUpOnPotsHang [PotsHang] (X)
          []
               LinkCw  !X  !FlashHook  !X  !Y ;
               exit 
     )
     endproc (* XGetsCwToneAndYHangsUp *)

     process XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw]
			     (X,Y:DecDigit)				(* E *)
     :exit:=
     (
        (
           LinkCw  !X  !HangsUp  !Y ;
           XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           PotsHang  !Y  !HangsUp ;
           XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           LinkCw  !X  !FlashHook ;
           (
              (
                 LinkCw  ?U:DecDigit  !NotOnLine  !X ;
                 CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
              )
             []
              (
                 LinkCw  ?U:DecDigit  !OnLine  !X ;
                 XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
              )
           )
        )
       []
        (
           LinkCw  !Y  !GetsCwTone ;
           XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
     )
     endproc (* XTalksAndYHolds *)

     process XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw]
			       (X,Y:DecDigit) 				(* F *)
     :exit:=
     (
        (
           LinkCw  !Y  !GetsCwTone ;
           XHangsUpAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           PotsHang  !Y  !HangsUp ;
           XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y)
        )
       []
        (
           PotsGate  !X  !RingsFrom  !Y ;
	   XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
     )
     endproc (* XHangsUpAndYHolds *)

     process XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] 
			       (X,Y:DecDigit) 				(* G *)
     :exit:=
     (
        LinkCw  !X  !FlashHook ;
        (
		LinkCw  ?U:DecDigit  !OnLine  !X ;
		XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (X,Y)
          []
		LinkCw  ?U:DecDigit  !NotOnLine  !X ;
		UserHangsUpOnPotsHang [PotsHang] (X)
        )
       []
        (
           LinkCw  !X  !HangsUp  !Y ;
           XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y) 
        )
     )
     endproc (* XTalksAndYHangsUp *)

     process XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y:DecDigit)	 (* H *)
     :exit:=
     (
	PotsGate  !X  !RingsFrom  !Y ;
	XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (X,Y)
     )
     endproc (* XandYHangUpWithYHolding *)

     process XRingsFromYwithYNotOnLine [PotsGate,PotsHang](X,Y:DecDigit) (* I *)
     :exit:=
     (
	   PotsGate  !X  !StopToRingFrom  !Y ;
	   exit
	[]
	   PotsGate  !X  !OffHookToAnswer !Y ;
	   UserHangsUpOnPotsHang [PotsHang] (X)
     )
     endproc (* XRingsFromYwithYNotOnLine *)

     process XandYGetCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y:DecDigit)
     :exit:=								 (* J *)
     (
        PotsConversation [PotsTalk]  (X,Y)
        [>
        (
           (
              PotsHang  !X  !HangsUp ;
              XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (Y,X)
            []
              PotsHang  !Y  !HangsUp ;
              XGetsCwToneAndYHangsUp [PotsHang,LinkCw] (X,Y)
           )
          []
           (
              LinkCw  !X  !FlashHook  !Y ;
              XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
            []
              LinkCw  !Y  !FlashHook  !X;
              XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
           )
        )
     )
     endproc (* XandYGetCwTone *)

     process XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] 
				  (X,Y:DecDigit)			 (* K *)
     :exit:=
     (
        (
           PotsHang  !Y  !HangsUp ;
           XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           LinkCw  !X  !FlashHook ;
           (
              LinkCw  ?U:DecDigit  !OnLine  !X ;
              XandYGetCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
            []
              LinkCw  ?U:DecDigit  !NotOnLine  !X ;
              XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
           )
        )
       []
        (
           LinkCw  !X  !HangsUp  !Y ;
           XHangsUpAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           LinkCw  !Y  !FlashHook  !X ;
           (
              LinkCw  !X  !HangsUp  !Y ;
              XHangsUpAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
           []
              LinkCw  !Y  !HangsUp  !X ;
              XHangsUpAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
           []
              LinkCw  !X  !FlashHook ;
              (
                 LinkCw  ?U:DecDigit  !NotOnLine  !X ;
                 XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
               []
                 LinkCw  ?U:DecDigit  !OnLine  !X ;
                 XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
              )
           []
              LinkCw  !Y  !FlashHook ;
              (
                 LinkCw  ?U:DecDigit  !NotOnLine  !Y ;
                 XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
              []
                 LinkCw  ?U:DecDigit  !OnLine  !Y ;
                 XTalksAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
              )
           )
        )
     )
     endproc (* XTalksAndYGetsCwTone *)

     process XHangsUpAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] 
				    (X,Y:DecDigit)			 (* L *)
     :exit:=
     (
        (
           PotsHang  !Y  !HangsUp ;
           XandYHangUpWithYHolding [PotsGate,PotsHang] (Y,X) 
        )
       []
        (
           LinkCw  !Y  !FlashHook  !X ;
           XHangsUpAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
       []
        (
           PotsGate  !X  !RingsFrom  !Y ;
           XRingsFromYAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
     )
     endproc (* XHangsUpAndYGetsCwTone *)

     process XRingsFromYAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] 
				       (X,Y:DecDigit)			 (* M *)
     :exit:=
     (
        (
           PotsHang  !Y  !HangsUp ;
           XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (Y,X) 
        )
        []
        (
           LinkCw  !Y  !FlashHook  !X ;
           XRingsFromYAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
        )
        []
        (
           PotsGate  !X  !OffHookToAnswer  !Y ;
           XGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
        )
     )
     endproc (* XRingsFromYAndYGetsCwTone *)

     process XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,LinkCw] 
				     (X,Y:DecDigit)			 (* N *)
     :exit:=
     (
	(
	   PotsHang  !Y  !HangsUp ;
	   XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (X,Y)
	)
	[]
	(
	   LinkCw  !Y  !GetsCwTone ;
	   XRingsFromYAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	)
	[]
	(
	   PotsGate  !X  !OffHookToAnswer  !Y ;
	   CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	)
     )
     endproc (* XRingsFromYwithYHolding *)

     process XHangsUpAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] 
			       (X,Y:DecDigit)				 (* O *)
     :exit:=
     (
	(
	   LinkCw  !Y  !FlashHook ;
	   (
		  LinkCw  ?U:DecDigit  !NotOnline  !Y ;
		  XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	      []
		  LinkCw  ?U:DecDigit  !OnLine  !Y ;
		  XHangsUpAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw] 
					 (X,Y)
	   )
	)
	[]
	(
	   LinkCw  !Y  !HangsUp  !X ;
	   (
		  PotsGate  !X  !RingsFrom  !Y ;
		  XRingsFromYAndYHangsUp [PotsGate,PotsTalk,PotsHang,LinkCw] 
					 (X,Y)
	      []
		  PotsGate  !Y  !RingsFrom  !X ;
		  XRingsFromYAndYHangsUp [PotsGate,PotsTalk,PotsHang,LinkCw] 
					 (Y,X)
	   )
	)
	[]
	(
	   PotsGate  !X  !RingsFrom  !Y ;
	   XRingsFromYAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	)
     )
     endproc (* XHangsUpAndYTalks *)

     process XRingsFromYAndYTalks [PotsGate,PotsTalk,PotsHang,LinkCw] 
				  (X,Y:DecDigit)			 (* P *)
     :exit:=
     (
	(
	   PotsGate  !X  !OffHookToAnswer  !Y ;
	   XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (Y,X)
	)
	[]
	(
	   PotsGate  !X  !StopToRingFrom  !Y ;
	   XTalksAndYHangsUp [PotsGate,PotsHang,LinkCw] (Y,X)
	)
	[]
	(
	   LinkCw  !Y  !HangsUp  !X ;
	   XRingsFromYAndYHangsUp [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	)
	[]
	(
	   LinkCw  !Y  !FlashHook ;
	   (
		  LinkCw  ?U:DecDigit  !NotOnLine  !Y ;
		  XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,LinkCw] 
					  (X,Y)
	      []
		  LinkCw  ?U:DecDigit  !OnLine  !Y ;
		  XRingsFromYAndYGetsCwTone [PotsGate,PotsTalk,PotsHang,LinkCw]
					    (X,Y)
	   )
	)
     )
     endproc (* XRingsFromYAndYTalks *)

     process XRingsFromYAndYHangsUp [PotsGate,PotsTalk,PotsHang,LinkCw] 
				    (X,Y:DecDigit)			 (* Q *)
     :exit:=
     (
	(
	   PotsGate  !X  !StopToRingFrom  !Y ;
	   XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y)
	)
	[]
	(
	   PotsGate  !X  !OffHookToAnswer  !Y ;
	   XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,LinkCw] (X,Y)
	)
	[]
	(
	   PotsGate  !Y  !RingsFrom  !X ;
	   (
	      (
		 PotsGate  !X  !OffHookToAnswer  !Y ;
		 XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,LinkCw]
					 (X,Y)
	      )
	     []
	      (
		  PotsGate  !Y  !OffHookToAnswer  !X ;
		  XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,LinkCw] 
					  (Y,X)
	      )
	     []
	      (
		  PotsGate  !X  !StopToRingFrom  !Y ;
		  XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (X,Y)
	      )
	     []
	      (
		  PotsGate  !Y  !StopToRingFrom  !X ;
		  XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (Y,X)
	      )
	   )
	)
     )
     endproc (* XRingsFromYAndYHangsUp *)

     process NewGlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
     :noexit:=
     (
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                 ( {} of DecSet ,
                   Insert (2,Insert (4, {} of DecSet)),
                   {} of SetOfPairs ,
                   {} of SetOfPairs ,
                   {} of SetOfPairs
                 )
        |[LinkCw]|

         GlobalCwController [LinkCw]
     )
     endproc (* NewGlobalView *)

     process GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
            ( BusyList ,              (* list of busy users               *)
              CwList      : DecSet ,  (* list of users with CallWaiting   *)
              WaitPairs,              (* list of pairs (Caller,Called)    *)
                                      (* where Called is waiting for X	  *)
                                      (* to answer			  *)
              HoldPairs,              (* list of pairs (X,Y) where X made *)
                                      (* a FlashHook and Y is holding     *)
              CwPairs     :SetOfPairs (* list of pairs (T,X) where T is the
                                         'CallWaiting' caller and X is the
                                         'CallWaiting' called             *)
            )
     :noexit:=
     (
             PotsPhases [PotsGate,PotsTalk,PotsHang,LinkCw]
                            ( BusyList ,
                              CwList   ,
                              WaitPairs,
                              HoldPairs,
                              CwPairs
                            )
         []  CwPhases [PotsGate,PotsTalk,PotsHang,LinkCw]
                            ( BusyList ,
                              CwList   ,
                              WaitPairs,
                              HoldPairs,
                              CwPairs
                            )
         []  PrintSets [PotsGate,PotsTalk,PotsHang,LinkCw]
                            ( BusyList ,
                              CwList   ,
                              WaitPairs,
                              HoldPairs,
                              CwPairs
                            )
     )
     where

     process PotsPhases [PotsGate,PotsTalk,PotsHang,LinkCw]
                        ( BusyList ,
                          CwList      : DecSet ,
                          WaitPairs,
                          HoldPairs,
                          CwPairs     : SetOfPairs
                        )
     :noexit:=
     (
        PotsGate  ?Caller : DecDigit  !OffHookToCall
        [     (Caller NotIn BusyList)
          and (SecondElement( Caller , WaitPairs ) eq none)
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( Insert (Caller,BusyList) ,
                     CwList   ,
                     InsertPair (Pair (Caller,Caller),WaitPairs),
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Caller : DecDigit  ! GetsTone  ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Caller : DecDigit  ! Dials  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Caller : DecDigit  ! GetsBusySignalFrom  ?Called : DecDigit
        [     (    (Called IsIn BusyList)
                or (    (SecondElement(Called,WaitPairs) ne none)
                    and (SecondElement(Called,WaitPairs) ne Caller)
                   )
                or (     (SecondElement(Called,HoldPairs) ne none)
                     and (SecondElement(Called,HoldPairs) ne none)
                   )
              )
          and (    (Called NotIn BusyList)
                or (Called eq Caller)
                or (Called NotIn CwList)
                or (FirstElement(Called,WaitPairs) ne none)
                or (SecondElement(Called,CwPairs) ne none)
              )
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Called  : DecDigit  ! RingsFrom  ?Caller : DecDigit
        [     (Called NotIn BusyList)
          and (    (SecondElement(Called,WaitPairs) eq none)
                or (SecondElement(Called,WaitPairs) eq Caller)
              )
          and (    (SecondElement(Called,HoldPairs) eq none)
                or (SecondElement(Called,HoldPairs) eq Caller)
              )
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     InsertPairAtTheEnd( Pair(Called,Caller) ,
                     RemovePair ( Pair (Caller,Caller),WaitPairs)),
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Caller : DecDigit  ! RingsBackFrom  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
    []  PotsGate  ?Called : DecDigit  ! OffHookToAnswer  ?Caller : DecDigit ;
        (
           [Caller eq SecondElement(Called,CwPairs)] 
	   -> GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( Insert (Called,BusyList),
                     CwList   ,
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     RemovePair (Pair (Called,Caller) , CwPairs )
                   )
	   []
           [Caller ne SecondElement(Called,CwPairs)] 
	   -> GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( Insert (Called,BusyList),
                     CwList   ,
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     CwPairs
                   )
        )
    []  PotsGate  ?Called :DecDigit  !StopToRingFrom  ?Caller :DecDigit
        [    (Caller NotIn BusyList)
          or (SecondElement(Caller,HoldPairs) eq Called)] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     RemovePair (Pair (Called,Caller) , CwPairs )
                   )
    []  PotsTalk  ?Caller : DecDigit  !TalksTo  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
    []  PotsHang  ?X : DecDigit  !HangsUp
        [SecondElement(X,HoldPairs) eq none] ;
        HangUpPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                    ( X,
                      Remove (X,BusyList),
                      CwList ,
                      RemovePair (Pair(FirstElement (X,WaitPairs),X),WaitPairs),
                      HoldPairs,
                      CwPairs
                    )
    )
    endproc (* PotsPhases *)

    process CwPhases [PotsGate,PotsTalk,PotsHang,LinkCw]
                     ( BusyList ,
                       CwList      : DecSet ,
                       WaitPairs,
                       HoldPairs,
                       CwPairs     : SetOfPairs)
    :noexit:=
    (
        LinkCw  ?Called:DecDigit   !GetsCwTone  ?Caller:DecDigit
        [    (Called IsIn BusyList)
         and (Called ne Caller)
         and (Called IsIn CwList)
         and (FirstElement(Called,WaitPairs) eq none)
         and (SecondElement(Called,CwPairs) eq none)
        ] ;
        LinkCw  !Called   !GetsCwTone ;
        GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( BusyList ,
                     CwList   ,
                     InsertPairAtTheEnd (Pair (Called,Caller),
                     RemovePair (Pair (Caller,Caller),WaitPairs)),
                     InsertPairAtTheEnd (Pair (Called,Caller),HoldPairs),
                     InsertPairAtTheEnd (Pair (Called,Caller),CwPairs)
                   )
   []   LinkCw  ?X:DecDigit  !HangsUp  ?U:DecDigit
        [SecondElement (X,HoldPairs) eq U] ;
        PotsHang  !X   !HangsUp ;
        HangUpPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                     ( X,
                       Remove ( X , BusyList ) ,
                       CwList ,
                       RemovePair (Pair(FirstElement(X,WaitPairs),X),WaitPairs),
                       HoldPairs,
                       CwPairs
                     )
   []   LinkCw  ?X:DecDigit  !FlashHook ;
        (
           LinkCw  !X  !FlashHook  ?U:DecDigit [U IsIn BusyList];
           LinkCw  !U  ?S:Signal  !X 
	   [(S IsEq OnLine) or (S IsEq NotOnLine)] ;
           FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( X,U,
                     BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
        [] LinkCw  !X  !FlashHook  !X  ?U:DecDigit [U NotIn BusyList] ;
           LinkCw  !U  ?S:Signal  !X
           [(S IsEq OnLine) or (S IsEq NotOnLine)] ;
           FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                   ( X,X,
                     BusyList ,
                     CwList   ,
                     WaitPairs,
                     HoldPairs,
                     CwPairs
                   )
        )
     )
     endproc (* CwPhases *)

    process HangUpPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                         (X           : DecDigit,     (* !X !HangsUp *)
                          BusyList ,
                          CwList      : DecSet ,
                          WaitPairs,
                          HoldPairs,
                          CwPairs     : SetOfPairs
                         )
    :noexit:=
    (  let T:DecDigit = SecondElement (X,CwPairs) in
       (
          [    (T eq none)
            or (T eq (SecondElement(X,WaitPairs)))
            or (T eq (SecondElement(X,HoldPairs)))
          ]  ->  GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (
                              BusyList ,
                              CwList   ,
                              WaitPairs,
                              HoldPairs,
                              CwPairs
                            )
         []
          [     (T ne none)
            and (T ne SecondElement(X,WaitPairs))
            and (T ne SecondElement(X,HoldPairs))
          ]  ->  GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (
                              BusyList ,
                              CwList   ,
                              WaitPairs,
                              HoldPairs,
                              RemovePair (Pair (X,T) , CwPairs )
                            )
       )
    )
    endproc (* HangUpPhase1 *)

    process FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,LinkCw]
                           (X,                (* !X !FlashHook !U *)
                            U           : DecDigit ,
                            BusyList ,
                            CwList      : DecSet ,
                            WaitPairs,
                            HoldPairs,
                            CwPairs     : SetOfPairs
                           )
    :noexit:=
    (  let T:DecDigit = SecondElement(X,CwPairs) in
       (
          [    (T eq SecondElement(X,HoldPairs))
            or (     (T IsIn BusyList)
                 and (SecondElement(T,HoldPairs) ne X)
               )
          ] -> FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,LinkCw]
                ( X,U,
                  BusyList,
                  CwList  ,
                  RemovePair (Pair (X,SecondElement(X,WaitPairs)),WaitPairs),
                  RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                  CwPairs
                )
       [] [     (T ne SecondElement(X,HoldPairs))
            and (    (T NotIn BusyList)
                  or (SecondElement(T,HoldPairs) eq X)
                )
          ] -> FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,LinkCw]
                ( X,U,
                  BusyList,
                  CwList  ,
                  RemovePair (Pair (X,SecondElement(X,WaitPairs)),WaitPairs),
                  RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                  RemovePair (Pair (X,T),CwPairs )
                )
       )
    )
    endproc (* FlashHookPhase1*)

    process FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,LinkCw]
                           (X,                (* !X !FlashHook !U *)
                            U           : DecDigit ,
                            BusyList ,
                            CwList      : DecSet ,
                            WaitPairs,
                            HoldPairs,
                            CwPairs     : SetOfPairs
                           )
    :noexit:=
    (
        [    (U NotIn BusyList)
          or (U eq X)
        ] ->  GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                        (
                          BusyList ,
                          CwList   ,
                          WaitPairs,
                          HoldPairs,
                          CwPairs
                        )
       []
        [     (U IsIn BusyList)
          and (U ne X)
        ] ->  GlobalView [PotsGate,PotsTalk,PotsHang,LinkCw]
                        (
                          BusyList ,
                          CwList   ,
                          WaitPairs,
                          InsertPairAtTheEnd (Pair (X,U) , HoldPairs ),
                          CwPairs
                        )
    )
    endproc (* FlashHookPhase2 *)

    endproc (* GlobalView *)

    process GlobalCwController [LinkCw]
    :noexit:=
    (
        LinkCw  ?X:DecDigit  !GetsCwTone  ?T:DecDigit ;
        LinkCw  !X  !GetsCwTone ;
        (
            NewCwControllerTX [LinkCw] (T,X)
            |||
            GlobalCwController [LinkCw]
        )
    )
    endproc (* GlobalCwController *)

    process NewCwControllerTX [LinkCw] (T,X : DecDigit )
    :noexit:=
    (
       (
          LinkCw  !X  !HangsUp  !T ;
          stop
       )
      []
       (
          LinkCw  !X  !FlashHook ;
          (
             (
                LinkCw  !X  !FlashHook  !X  ?Y:DecDigit [Y ne T] ;
                LinkCw  ?U:DecDigit  !NotOnLine  !X;
                stop
             )
            []
             (
                LinkCw  !X  !FlashHook  ?Y:DecDigit [Y ne X] ;
                LinkCw  ?U:DecDigit  !OnLine  !X ;
                CwControllerTX_TTalksToX [LinkCw] (T,X,Y)
             )
          )
       )
    )
    endproc (* NewCwControllerTX *)

    process CwControllerTX_TTalksToX [LinkCw] (T,X,Y:DecDigit)
    :noexit:=
    (
       (
          LinkCw  !X  !HangsUp  ?U:DecDigit [U ne T] ;
          stop
       )
      []
       (
          LinkCw  !X  !FlashHook ;
          (
             (
                LinkCw  !X  !FlashHook  !X  !T ;
                LinkCw  ?U:DecDigit  !NotOnLine  !X ;
                stop
             )
            []
             (
                LinkCw  !X  !FlashHook  !T ;
                LinkCw  ?U:DecDigit  !OnLine  !X ;
                NewCwControllerTX [LinkCw] (T,X)
             )
          )
       )
    ) endproc (* CwControllerTX_TTalksToX *)

    process PrintSets [PotsGate,PotsTalk,PotsHang,LinkCw]
                      ( B, 
                        Cw      : DecSet ,
                        Wpairs,
                        HPairs,
                        Cwpairs : SetOfPairs
                      )
    :noexit:=
    (   hide BusyList, CwList, WaitPairs, HoldPairs, CwPairs  in
        (
                BusyList    !B;
                GlobalView  [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (B,Cw,Wpairs,Hpairs,Cwpairs)
              []
                CwList    !Cw;
                GlobalView  [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (B,Cw,Wpairs,Hpairs,Cwpairs)
              []
                WaitPairs    !Wpairs;
                GlobalView  [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (B,Cw,Wpairs,Hpairs,Cwpairs)
              []
                HoldPairs    !Hpairs;
                GlobalView  [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (B,Cw,Wpairs,Hpairs,Cwpairs)
              []
                CwPairs    !Cwpairs;
                GlobalView  [PotsGate,PotsTalk,PotsHang,LinkCw]
                            (B,Cw,Wpairs,Hpairs,Cwpairs)
       )
    )
    endproc

endspec
 

