(*----------------------------------------------------------------------------
	Author: Edouard Hibon
	Updated on: November 1994.
------------------------------------------------------------------------------*)
	
specification Twc_Cw [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
	
:noexit
	
library Set, Boolean, DecDigit, NaturalNumber, BasicNaturalNumber endlib
	
type UsersList is DecDigit, Boolean, BasicNaturalNumber
sorts List
	
   opns {} : -> List
	InsertL, RemoveL			: DecDigit, List -> List
	_IsIn_, _NotIn_ 			: DecDigit, List -> Bool
	_Union_, _Ints_, _Minus_ 		: List, List     -> List
	_eq_, _ne_, _Includes_, _IsSubsetOf_ 	: List, List 	 -> Bool
	Card 					: List 		 -> Nat
	First 					: List 		 -> DecDigit
	
   eqns  forall x, y : DecDigit,
		s, t : List
	
   ofsort List
	x IsIn InsertL(y,s) =>
	InsertL(x, InsertL(y,s)) = InsertL(y,s) ;
	RemoveL(x, {}) = {} ;
	RemoveL(x, InsertL(x,s)) = s ;
	x ne y =>
	RemoveL(x,InsertL(y,s)) = InsertL(y,RemoveL(x,s)) ;
	{} Union s = s ;
	InsertL(x,s) Union t = InsertL(x,s Union t) ;
	{} Ints s = {} ; 
	x IsIn t =>
	InsertL(x,s) Ints t = InsertL(x,s Ints t) ;
	x NotIn t =>
	InsertL(x,s) Ints t = s Ints t ;
	s Minus {} = s ;
	s Minus InsertL(x, t) = RemoveL(x,s) Minus t ;
	
   ofsort Bool
	x IsIn {} = false ;
	x eq y =>
	x IsIn InsertL(y,s) = true ; 
	x ne y =>
	x IsIn InsertL(y,s) = x IsIn s ;
	x NotIn s = not(x IsIn s) ;
	s Includes {} = true ;
	s Includes InsertL(x,t) = (x IsIn s) and (s Includes t) ;
	s IsSubsetOf t = t Includes s ;
	s eq t = (s Includes t) and (t Includes s);
	s ne t = not(s eq t) ;

   ofsort DecDigit
	First({}) = 0 ;
	x NotIn s =>
	First(InsertL(x,s)) = x ;
	ofsort Nat
	Card ({}) = 0 ;
	x NotIn s =>
	Card (InsertL(x,s)) = Succ(Card(s)) ;
	endtype
	
type EventSeq is NaturalNumber, Boolean
sorts EventSeq

   opns 0, 1, 2, 3, 4, 5, 6, 7, 8, 9       : 			-> EventSeq
	_eq_, _ne_, _lt_, _le_, _ge_, _gt_ : EventSeq, EventSeq -> Bool
	NatNum 				   : EventSeq 		-> Nat
	
   eqns forall x, y, z : EventSeq
		
   ofsort Nat
	NatNum(0) = 0               ;
	NatNum(1) = Succ(NatNum(0)) ;
	NatNum(2) = Succ(NatNum(1)) ;
	NatNum(3) = Succ(NatNum(2)) ;
	NatNum(4) = Succ(NatNum(3)) ;
	NatNum(5) = Succ(NatNum(4)) ;
	NatNum(6) = Succ(NatNum(5)) ;
	NatNum(7) = Succ(NatNum(6)) ;
	NatNum(8) = Succ(NatNum(7)) ;
	NatNum(9) = Succ(NatNum(8)) ;
	
   ofsort  Bool
	x eq y  = NatNum(x) eq NatNum(y) ;
	x ne y  = NatNum(x) ne NatNum(y) ;
	x lt y  = NatNum(x) lt NatNum(y) ;
	x le y  = NatNum(x) le NatNum(y) ;
	x ge y  = NatNum(x) ge NatNum(y) ;
	x ge y  = NatNum(x) ge NatNum(y) ;
	
endtype
	
type DigitPair is DecDigit, Boolean
sorts DigitPair

   opns
	None    : -> DecDigit
	Pair:DecDigit,DecDigit -> DigitPair
	_eq_: DigitPair, DigitPair -> Bool
	
   eqns forall f1,f2,l1,l2:DecDigit , f3:Nat
	
   ofsort Bool
	f1 eq None     = false;
	None     eq f1 = false;
	Pair(f1,l1) eq Pair(f2,l2) = (f1 eq f2) and (l1 eq l2);
	
   ofsort Nat
	NatNum(None    ) =  0
	endtype
	
type SetOfPairs  is DigitPair, NaturalNumber
sorts SetOfPairs

   opns
	{}:                                          -> SetOfPairs
	InsertPair:        DigitPair,SetOfPairs  -> SetOfPairs
	RemovePair:        DigitPair,SetOfPairs  -> SetOfPairs
	InsertPairAtTheEnd:DigitPair,SetOfPairs  -> SetOfPairs
	IsPairIn :         DigitPair,SetOfPairs  -> Bool
	SecondElement:     DecDigit,SetOfPairs   -> DecDigit
	(* given the first element, find the second element *)
	FirstElement:      DecDigit,SetOfPairs   -> DecDigit
	(* given the secondelement, find the first element *)
	IsPairNotIn:       DigitPair,SetOfPairs  -> Bool
	CardPair:          DecDigit,SetOfPairs   -> Nat
	(* Number of pairs with the same first element Insert the set *)
	
   eqns forall x,y: DigitPair,
	s: SetOfPairs,
	f1, f2, s1, s2: DecDigit
	
   ofsort SetOfPairs
	InsertPairAtTheEnd(x,{})              = InsertPair(x,{});
	InsertPairAtTheEnd(x,InsertPair(x,s)) = InsertPair(x,s) ;
	not(x eq y) =>
	InsertPairAtTheEnd(x,InsertPair(y,s)) =
	InsertPair(y,InsertPairAtTheEnd(x,s));
	
	RemovePair(x,{}) = {};
	RemovePair(x,InsertPair(x,s)) = s;
	not(x eq y) =>
	RemovePair(x,InsertPair(y,s)) = InsertPair(y,RemovePair(x,s));
	
   ofsort Bool
	IsPairIn(x,{}) = false;
	IsPairIn(x,InsertPair(y,s)) = (x eq y) or (IsPairIn(x,s));
	IsPairIn(x,RemovePair(x,s)) = false;
	not(x eq y) =>
	IsPairIn(x,RemovePair(y,s)) = IsPairIn(x,s);
	IsPairNotIn(x,s) = not(IsPairIn(x,s));
	
   ofsort DecDigit
	SecondElement (f1,{}) = None     ;
	SecondElement (f1,InsertPair(Pair(f1,s1),s)) = s1;
	f1 ne f2 =>
	SecondElement (f1,InsertPair(Pair(f2,s2),s)) = SecondElement (f1,s);
	FirstElement (f1,{}) = None     ;     (* not tested *)
	FirstElement (s1,InsertPair(Pair(f1,s1),s)) = f1;
	s1 ne s2 =>
	FirstElement (s1,InsertPair(Pair(f2,s2),s)) = FirstElement (s1,s);
	
   ofsort  Nat
	CardPair (f1, {})                = 0;
	IsPairNotIn(Pair(f2, s1), s) and (f1 eq f2) =>
	CardPair (f1, InsertPair(Pair(f2, s1), s)) = Succ(CardPair(f1,s));
	IsPairNotIn(Pair(f2, s1), s) and not(f1 eq f2) =>
	CardPair (f1, InsertPair(Pair(f2, s1), s)) = CardPair(f1,s);
	endtype
	
type DecSet0 is Set
   actualizedby DecDigit
   using sortnames  DecDigit for Element
		        Bool for FBool
endtype
	
type DecSet is DecSet0
   renamedby sortnames  DecSet for Set
endtype
	
type Signal is Boolean, NaturalNumber
sorts  Signal

   opns OffHookToCall,
	OffHookToAnswer,
	HangsUp,
	TalksTo,
	Rings,
	StopToRing,
	RingsFrom,
	StopToRingFrom,
	RingsBackFrom,
	Dials,
	GetsTone,
	GetsBusySignalFrom,
	GetsCwTone,
	FlashHook,
	NotOnLine,
	OnLine,
	YandTOnLine,
	YandTNotOnLine,
	YNotOnLine,
	TNotOnLine,
	TMustHangUp     :-> Signal
	
	_IsEq_          : Signal, Signal -> Bool
	_IsNe_          : Signal, Signal -> Bool
	Map             : Signal -> Nat
	
   eqns forall x, y: Signal
	
   ofsort  Nat
	Map (OffHookToCall)       =  0;
	Map (OffHookToAnswer)     =  Succ(Map (OffHookToCall));
	Map (HangsUp)             =  Succ(Map (OffHookToAnswer));
	Map (TalksTo)             =  Succ(Map (HangsUp));
	Map (Rings)               =  Succ(Map (TalksTo));
	Map (StopToRing)          =  Succ(Map (Rings));
	Map (RingsFrom)           =  Succ(Map (StopToRing));
	Map (StopToRingFrom)      =  Succ(Map (RingsFrom));
	Map (RingsBackFrom)       =  Succ(Map (StopToRingFrom));
	Map (Dials)               =  Succ(Map (RingsBackFrom));
	Map (GetsTone)            =  Succ(Map (Dials));
	Map (GetsBusySignalFrom)  =  Succ(Map (GetsTone));
	Map (GetsCwTone)          =  Succ(Map (GetsBusySignalFrom));
	Map (FlashHook)           =  Succ(Map (GetsCwTone));
	Map (NotOnLine)           =  Succ(Map (FlashHook));
	Map (OnLine)              =  Succ(Map (NotOnLine));
	Map (YandTOnLine)         =  Succ(Map (OnLine));
	Map (YandTNotOnLine)      =  Succ(Map (YandTOnLine));
	Map (YNotOnLine)          =  Succ(Map (YandTNotOnLine));
	Map (TNotOnLine)          =  Succ(Map (YNotOnLine));
	Map (TMustHangUp)         =  Succ(Map (TNotOnLine));
	
   ofsort  Bool
	x IsEq y = Map(x) eq Map(y);
	x IsNe y = Map(x) ne Map(y);
	endtype
	
	
behaviour
(
    Twc_Cw [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
)
	
where

process Twc_Cw [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
:noexit:=
(
    InitConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
    ||
    NewGlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
)
endproc (* Twc_Cw *)

process InitConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
:noexit:=
(
    InitNewConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
    |||
    i; InitConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
)
endproc (* InitConnection *)

process InitNewConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
:noexit:=
(
    PotsGate  ?Caller:DecDigit   !OffHookToCall ;
    (
	CallerGetsTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (Caller)
	|[TwcLink]|
	TwcConnection [PotsGate,PotsTalk,PotsHang,TwcLink] (Caller)
	)
    >> InitNewConnection [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
)
endproc (* InitNewConnection *)

process CallerGetsTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                       (Caller:DecDigit)
:exit:=
(
    PotsGate  !Caller  !GetsTone ;
    (
	PotsGate  !Caller  !Dials  ?Called:DecDigit ;
	(
	    CalledRingsFromCaller [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
				  (Caller,Called)
	  []
	    CalledGetsCwToneFromCaller [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
					(Caller,Called)
	  []
	    CallerGetsBusySignalFromCalled [PotsGate,PotsHang]
					   (Caller,Called)
	  []
	    UserHangsUp [PotsHang] (Caller)
	)
	[]  UserHangsUp [PotsHang] (Caller)
    )
    []  UserHangsUp [PotsHang] (Caller)
)
endproc (* InitNewConnection *)

process UserHangsUp [Gate] (User:DecDigit)
:exit:=
(
    Gate  !User  !HangsUp ;
    exit
)
endproc (* UserHangsUp *)

process CallerGetsBusySignalFromCalled [PotsGate,PotsHang]
					(Caller,Called:DecDigit)
:exit:=
(
    PotsGate  !Caller  !GetsBusySignalFrom  !Called ;
    UserHangsUp [PotsHang] (Caller)
)
endproc (* CallerGetsBusySignalFromCalled *)

process CalledRingsFromCaller [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			      (Caller,Called:DecDigit)
:exit:=
(
    PotsGate  !Called  !RingsFrom  !Caller ;
    (
	PotsGate  !Caller  !RingsBackFrom  !Called ;
	(
	    PotsGate  !Called  !OffHookToAnswer  !Caller ;
	    CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
				(Caller,Called)
	  []
	    UserHangsUp [PotsHang] (Caller)
	)
	[]  UserHangsUp [PotsHang] (Caller)
    )
)
endproc (* CalledRingsFromCaller *)

process CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]  (* B *)
                            (Caller,Called : DecDigit)
:exit:=
(
    Conversation [PotsTalk] (Caller,Called)
    [>
    (
	(
	    PotsHang  !Caller  !HangsUp ;
	    (
		UserHangsUp [PotsHang] (Called)
	      []
		CwLink  !Called  !GetsCwTone ;
		XGetsCwToneAndYHangsUp [PotsHang,CwLink] (Called,Caller)
	    )
	)
	[]
	(
	    PotsHang  !Called  !HangsUp ;
	    (
		UserHangsUp [PotsHang] (Caller)
	      []
		CwLink  !Caller  !GetsCwTone ;
		XGetsCwToneAndYHangsUp [PotsHang,CwLink] (Caller,Called)
	    )
	)
	[]
	(
	    CwLink  !Caller  !GetsCwTone ;
	    XGetsCwTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] 
			(Caller,Called)
	)
	[]
	(
	    CwLink  !Called  !GetsCwTone ;
	    XGetsCwTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] 
			(Called,Caller)
	)
	[]
	(
	    TwcLink  !Caller  !FlashHook  !Called ;
	    XCallsTandYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			     (Caller,Called)
	)
	[]
	(
	    TwcLink  !Called  !FlashHook  !Caller ;
	    XCallsTandYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			     (Called,Caller)
	)
    )
)
endproc (* CallerTalksToCalled *)

process Conversation [PotsTalk]  (Caller,Called:DecDigit)
:noexit:=
(
    PotsTalk  !Caller  !TalksTo  !Called ;
    Conversation [PotsTalk] (Caller,Called)
  []
    PotsTalk  !Called  !TalksTo  !Caller ;
    Conversation [PotsTalk] (Caller,Called)
)
endproc (* Conversation *)

process XCallsTandYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			 (X,Y : DecDigit )
:exit:=
(
    (
	TwcLink  !X  !FlashHook ;
	(
	      TwcLink  !TNotOnLine  !X ;
	      CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
				  (X,Y)
	  []
	      TwcLink  !YandTOnLine  !X ;
	     XTalksToYandT [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
	)
    )
    []
    (
	PotsHang  !Y  !HangsUp ;
	(
	    (
		TwcLink  !X  !FlashHook ;
		(
			 TwcLink  !YNotOnLine  !X ;
			 UserHangsUp [TwcLink] (X)
		    []
			 TwcLink  !YandTNotOnLine  !X ;
			 UserHangsUp [PotsHang] (X)
		)
	    )
	  []
	    (
		TwcLink  !X  !HangsUp ;
		PotsHang !X  !HangsUp ;
		PotsGate !X  !RingsFrom  !Y ;
		XRingsFromYandYNotOnLine [PotsGate,PotsHang] (X,Y)
	    )
	)
    )
    []
    (
	TwcLink  !X  !HangsUp ;
	PotsHang  !X   !HangsUp ;
	(
	    (
		PotsHang  !Y  !HangsUp ;
		PotsGate !X  !RingsFrom  !Y ;
		XRingsFromYandYNotOnLine [PotsGate,PotsHang] (X,Y)
	    )
	  []
	    (
		PotsGate !X  !RingsFrom  !Y ;
		(
		    (
			PotsHang  !Y  !HangsUp ;
			XRingsFromYandYNotOnLine [PotsGate,PotsHang] (X,Y)
		    )
		  []
		    (
			PotsGate  !X  !OffHookToAnswer  !Y ;
			CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,
					     CwLink] (X,Y)
		    )
		)
	    )
	)
    )
)
endproc (* XCallsTandYHolds *)

process XRingsFromYandYNotOnLine [PotsGate,PotsHang] (X,Y:DecDigit)
:exit:=
(
    (
	PotsGate  !X  !StopToRingFrom  !Y ;
	exit
    )
    []
    (
	PotsGate  !X  !OffHookToAnswer  !Y ;
	UserHangsUp [PotsHang] (X)
    )
)
endproc (* XRingsFromYandYNotOnLine *)

process XTalksToYandT [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y:DecDigit)
:exit:=
(
    Conversation [PotsTalk] (X,Y)
    [>
    (
	(
	    TwcLink  !TNotOnLine  !X ;
	    CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink](X,Y)
	)
	[]
	(
	    TwcLink  !X  !FlashHook ;
	    TwcLink  !TMustHangUp  !X ;
	    CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink](X,Y)
	)
	[]
	(
	    PotsHang  !Y  !HangsUp ;
	    TwcLink  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (X)
	)
	[]
	(
	    TwcLink  !X  !HangsUp ;
	    PotsHang  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (Y)
	)
    )
)
endproc (* XTalksToYandT *)

process TwcConnection [PotsGate,PotsTalk,PotsHang,TwcLink]
		      (Caller:DecDigit)
:exit:=
(
    TwcLink  ?X:DecDigit  !FlashHook  ?Y:DecDigit ;
    (
	TwcConnection [PotsGate,PotsTalk,PotsHang,TwcLink] (Caller)
	|||
	TwcNewConnection [PotsGate,PotsTalk,PotsHang,TwcLink] (X)
    )
    [] exit
)
endproc (* TwcConnection *)

process TwcNewConnection [PotsGate,PotsTalk,PotsHang,TwcLink] (X:DecDigit)
:exit:=
(
    PotsGate  !X  !GetsTone ;
    (
	PotsGate  !X  !Dials  ?T:DecDigit ;
	(
	    (
		PotsGate  !X  !GetsBusySignalFrom  !T ;
		XHangsUpOrXFlasHook [TwcLink]  (X)
	    )
	    []
	    (
		PotsGate  !T  !RingsFrom      !X ;
		(
		    PotsGate  !X  !RingsBackFrom  !T ;
		    (
			(
			    PotsGate  !T  !OffHookToAnswer  !X ;
			    XTalksToT [PotsTalk,PotsHang,TwcLink] (X,T)
			)
			[]  XHangsUpOrXFlasHook [TwcLink]  (X)
		    )
		   []  XHangsUpOrXFlasHook [TwcLink]  (X)
		)
	    )
	    []  XHangsUpOrXFlasHook [TwcLink]  (X)
	)
	[]  XHangsUpOrXFlasHook [TwcLink]  (X)
    )
    []  XHangsUpOrXFlasHook [TwcLink]  (X)
)
endproc (* TwcController *)

process XHangsUpOrXFlasHook [TwcLink] (X:DecDigit)
:exit:=
(
	UserHangsUp [TwcLink] (X)
    []
    (
	TwcLink  !X  !FlashHook ;
	(
	    TwcLink  !TNotOnLine  !X ;
	    exit
	[]
	    TwcLink  !YandTNotOnLine  !X ;
	    exit
	)
    )
)
endproc (* XHangsUpOrXFlasHook *)

process XTalksToT [PotsTalk,PotsHang,TwcLink] (X,T:DecDigit)
:exit:=
(
    Conversation [PotsTalk] (X,T)
    [>
    (
	(
	    PotsHang  !T !HangsUp;
	    XHangsUpOrXFlasHook [TwcLink] (X)
	)
	[]
	(
	    TwcLink  !X  !FlashHook ;
	    (
		TwcLink  !YNotOnLine  !X ;
		XTalksToTAlone [PotsTalk,PotsHang,TwcLink] (X,T)
	      []
		TwcLink  !YandTOnLine  !X ;
		XTalksToTandY [PotsTalk,PotsHang,TwcLink] (X,T)
	    )
	)
	[]
	(
	    TwcLink  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (T)
	)
    )
)
endproc (* XTalksToT *)

process XTalksToTAlone [PotsTalk,PotsHang,TwcLink] (X,T:DecDigit)
:exit:=
(
    Conversation [PotsTalk] (X,T)
    [>
    (
	(
	    TwcLink  !X  !HangsUp ;
	    PotsHang  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (T)
	)
	[]
	(
	    PotsHang  !T  !HangsUp ;
	    TwcLink  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (X)
	)
    )
)
endproc (* XTalksToTAlone *)

process XTalksToTandY [PotsTalk,PotsHang,TwcLink] (X,T:DecDigit)
:exit:=
(
    Conversation [PotsTalk ] (X,T)
    [>
    (
	(
	    TwcLink  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (T)
	)
	[]
	(
	    PotsHang  !T  !HangsUp  !X ;
	    TwcLink  !TNotOnLine  !X ;
	    exit
	)
	[]
	(
	    TwcLink  !X  !FlashHook ;
	    TwcLink  !TMustHangUp  !X ;
	    UserHangsUp [PotsHang] (T)
	)
    )
)
endproc (* XTalksToTandY *)


process CalledGetsCwToneFromCaller [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			              (Caller,Called : DecDigit)
:exit:=
(
    CwLink  !Called  !GetsCwTone  !Caller ;
    (
	(
	    PotsGate  !Caller  !RingsBackFrom  !Called ;
	    (
		(
		    CwLink  !Called  !FlashHook ;
		    (
			CwLink  ?U:DecDigit  !NotOnLine ;
			CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,
			                     CwLink] (Caller,Called)
		      []
			CwLink  ?U:DecDigit  !OnLine ;
			XGetsCwTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			            (Called,Caller)
		    )
		)
		[]
		(
		    CwLink  !Called  !HangsUp  !Caller ;
		    XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			            (Called,Caller)
		)
		[]
		(
		    PotsHang  !Caller  !HangsUp ;
		    XTalksAndYHangsUp [PotsGate,PotsHang,CwLink]
			            (Called,Caller)
		)
	    )
	)
	[]
	(
	    PotsHang  !Caller  !HangsUp ;
	    XTalksAndYHangsUp [PotsGate,PotsHang,CwLink]
			      (Called,Caller)
	)
	[]
	(
	    CwLink  !Called  !HangsUp  !Caller ;
	    (
		(
		    PotsHang  !Caller  !HangsUp ;
		    XandYHangUpWithYHolding [PotsGate,PotsHang] (Called,Caller)
		)
		[]
		(
		    PotsGate  !Caller  !RingsBackFrom  !Called ;
		    XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,
				       CwLink] (Called,Caller)
		)
		[]
		(
		    PotsGate  !Called  !RingsFrom  !Caller ;
		    (
			PotsHang  !Caller  !HangsUp ;
			XRingsFromYwithYNotOnLine [PotsGate,PotsHang]
			                          (Called,Caller)
		      []
			PotsGate  !Caller  !RingsBackFrom  !Called ;
			XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,
						 TwcLink,CwLink] (Called,Caller)
		    )
		)
	    )
	)
    )
)
endproc (* CalledGetsCwToneFromCaller *)

process XGetsCwTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink](X,Y:DecDigit)
:exit:=									(* C *)
(
    Conversation [PotsTalk]  (X,Y)
    [>
    (
	 (
	    PotsHang  !X  !HangsUp ;
	    UserHangsUp [PotsHang] (Y)
	 )
	 []
	 (
	    PotsHang  !Y  !HangsUp ;
	    XGetsCwToneAndYHangsUp [PotsHang,CwLink] (X,Y)
	 )
	 []
	 (
	    CwLink  !X  !FlashHook  !Y ;
	    XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
	)
    )
)
endproc (* PotsControlXGetsCwTone *)

process XGetsCwToneAndYHangsUp [PotsHang,CwLink] (X,Y : DecDigit)	 (* D *)
:exit:=
(
     UserHangsUp [PotsHang] (X)
  []
     CwLink  !X  !FlashHook  !X  !Y ;
     exit
)
endproc (* XGetsCwToneAndYHangsUp *)

process XTalksAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			(X,Y:DecDigit)                             	(* E *)
:exit:=
(
    (
	CwLink  !X  !HangsUp  !Y ;
	XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
    )
   []
    (
	PotsHang  !Y  !HangsUp ;
	XTalksAndYHangsUp [PotsGate,PotsHang,CwLink] (X,Y)
    )
   []
    (
	CwLink  !X  !FlashHook ;
	(
	    (
		CwLink  ?U:DecDigit  !NotOnLine ;
		CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
				     (X,Y)
	    )
	    []
	    (
		CwLink  ?U:DecDigit  !OnLine ;
		XGetsCwTone [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
	    )
	)
    )
)
endproc (* XTalksAndYHolds *)

process XHangsUpAndYHolds [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			     (X,Y:DecDigit)                           (* F *)
:exit:=
(
    (
	PotsHang  !Y  !HangsUp ;
	XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y)
    )
    []
    (
	PotsGate  !X  !RingsFrom  !Y ;
	XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
    )
)
endproc (* XHangsUpAndYHolds *)

process XTalksAndYHangsUp [PotsGate,PotsHang,CwLink]
			     (X,Y:DecDigit)                           (* G *)
:exit:=
(
    CwLink  !X  !FlashHook ;
    (
	CwLink  ?U:DecDigit  !OnLine ;
	XGetsCwToneAndYHangsUp [PotsHang,CwLink] (X,Y)
      []
	CwLink  ?U:DecDigit  !NotOnLine ;
	UserHangsUp [PotsHang] (X)
    )
    []
    (
	CwLink  !X  !HangsUp  !Y ;
	XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y)
    )
)
endproc (* XTalksAndYHangsUp *)

process XandYHangUpWithYHolding [PotsGate,PotsHang] (X,Y:DecDigit)  (* H *)
:exit:=
(
    PotsGate  !X  !RingsFrom  !Y ;
    XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (X,Y)
)
endproc (* XandYHangUpWithYHolding *)

process XRingsFromYwithYNotOnLine [PotsGate,PotsHang](X,Y:DecDigit) (* I *)
:exit:=
(
	PotsGate  !X  !StopToRingFrom  !Y ;
	exit
     []
	PotsGate  !X  !OffHookToAnswer !Y ;
	UserHangsUp [PotsHang] (X)
)
endproc (* XRingsFromYwithYNotOnLine *)

process XRingsFromYwithYHolding [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
			        (X,Y:DecDigit)                           (* N *)
:exit:=
(
    (
	PotsHang  !Y  !HangsUp ;
	XRingsFromYwithYNotOnLine [PotsGate,PotsHang] (X,Y)
    )
    []
    (
	PotsGate  !X  !OffHookToAnswer  !Y ;
	CallerTalksToCalled [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink] (X,Y)
    )
)
endproc (* XRingsFromYwithYHolding *)


process NewGlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
:noexit:=
(
    GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                 (
                   {} of DecSet     ,
                   {} of SetOfPairs ,
                   {} of SetOfPairs ,
                   Insert (2,{} of DecSet) ,
                   {} of SetOfPairs ,
                   Insert (2,{} of DecSet) ,
                   {} of DecSet
                 )
    |[CwLink]|

    GlobalCwController [CwLink]
)
endproc (* NewGlobalView *)

process GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
            (
              BusyList   : DecSet ,     (* list of busy users               *)
              WaitPairs,                (* list of pairs (Caller,Called)    *)
                                        (* where Called is waiting for X    *)
                                        (* to answer                        *)
              HoldPairs  : SetofPairs,  (* list of pairs (X,Y) where X made *)
                                        (* a FlashHook and Y is holding     *)
              CwList     : DecSet ,     (* list of users with Cw_feature    *)
              CwPairs    : SetOfPairs,  (* list of pairs (T,X) where T is the
                                         'CallWaiting' caller and X is the
                                         'CallWaiting' called               *)
              TwcList ,                 (* list of users with Twc_feature   *)
              TwcUsers   :DecSet        (* list of users using their Twc_
                                           feature                          *)
            )
:noexit:=
(
       PotsPhases [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                  (
                    BusyList ,
                    WaitPairs,
                    HoldPairs,
                    CwList   ,
                    CwPairs  ,
                    TwcList  ,
                    TwcUsers
                  )
   []  CwPhases  [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                  (
                    BusyList ,
                    WaitPairs,
                    HoldPairs,
                    CwList   ,
                    CwPairs  ,
                    TwcList  ,
                    TwcUsers
                  )
   []  TwcPhases  [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                  (
                    BusyList ,
                    WaitPairs,
                    HoldPairs,
                    CwList   ,
                    CwPairs  ,
                    TwcList  ,
                    TwcUsers
                  )
)
where

     process PotsPhases [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                        (
                          BusyList    : DecSet ,
                          WaitPairs,
                          HoldPairs   : SetOfPairs ,
                          CwList      : DecSet ,
                          CwPairs     : SetOfPairs ,
                          TwcList ,
                          TwcUsers    :DecSet
                        )
     :noexit:=
     (
        PotsGate  ?Caller : DecDigit  !OffHookToCall
        [     (Caller NotIn BusyList)
          and (SecondElement( Caller , WaitPairs ) eq none)
          and (SecondElement( Caller , HoldPairs ) eq none)
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( Insert (Caller,BusyList) ,
                     InsertPair (Pair (Caller,Caller),WaitPairs),
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Caller : DecDigit  ! GetsTone  ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Caller : DecDigit  ! Dials  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Caller : DecDigit  ! GetsBusySignalFrom  ?Called : DecDigit
        [     (    (Called IsIn BusyList)
                or (    (SecondElement(Called,WaitPairs) ne none)
                    and (SecondElement(Called,WaitPairs) ne Caller)
                   )
                or (     (SecondElement(Called,HoldPairs) ne none)
                     and (SecondElement(Called,HoldPairs) ne none)
                   )
              )
          and (    (Called NotIn BusyList)
                or (Called eq Caller)
                or (Called NotIn CwList)
                or (FirstElement(Called,WaitPairs) ne none)
                or (SecondElement(Called,CwPairs) ne none)
		or (Called IsIn TwcUsers)
              )
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Called  : DecDigit  ! RingsFrom  ?Caller : DecDigit
        [     (Called NotIn BusyList)
          and (    (SecondElement(Called,WaitPairs) eq none)
                or (SecondElement(Called,WaitPairs) eq Caller)
              )
          and (    (SecondElement(Called,HoldPairs) eq none)
                or (SecondElement(Called,HoldPairs) eq Caller)
              )
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     InsertPairAtTheEnd( Pair(Called,Caller) ,
                      RemovePair ( Pair (Caller,Caller),WaitPairs)),
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Caller : DecDigit  ! RingsBackFrom  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsGate  ?Called : DecDigit  ! OffHookToAnswer  ?Caller : DecDigit ;
        (
           [     (Caller eq SecondElement(Called,CwPairs))
             and (    (Caller NotIn BusyList)
                   or (SecondElement(Caller,HoldPairs) eq Called)
                   or (SecondElement(Called,HoldPairs) eq none)
                 )
           ] ->  GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( Insert (Called,BusyList),
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     CwList   ,
                     RemovePair (Pair (Called,Caller) , CwPairs )   ,
                     TwcList  ,
                     TwcUsers
                   )
           []
           [    (Caller ne SecondElement(Called,CwPairs))
             or (     (Caller IsIn BusyList)
                  and (SecondElement(Caller,HoldPairs) ne Called)
                  and (SecondElement(Called,HoldPairs) eq none)
                )
           ] ->   GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( Insert (Called,BusyList),
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
        )
    []  PotsGate  ?Called :DecDigit  !StopToRingFrom  ?Caller :DecDigit
        [    (Caller NotIn BusyList)
          or (SecondElement(Caller,HoldPairs) eq Called)] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     RemovePair (Pair (Called,Caller) , WaitPairs ) ,
                     RemovePair (Pair (Called,Caller) , HoldPairs ) ,
                     CwList   ,
                     RemovePair (Pair (Called,Caller) , CwPairs )   ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsTalk  ?Caller : DecDigit  !TalksTo  ?Called : DecDigit ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsHang  ?X : DecDigit  !HangsUp
        [     (X NotIn TwcUsers)
          and (SecondElement(X,HoldPairs) eq none)
        ] ;
        HangUpPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( X,
                     Remove (X,BusyList),
                     RemovePair (Pair(FirstElement (X,WaitPairs),X),WaitPairs),
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
    []  PotsHang  ?T:DecDigit  !HangsUp  ?X:DecDigit
        [T NotIn TwcUsers] ;
        TwcLink  !TNotOnLine  !X ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     Remove (T,BusyList),
                     RemovePair (Pair (FirstElement (T,WaitPairs), T),
                      WaitPairs ) ,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     Remove (X,TwcUsers)
                   )
    )
    endproc (* PotsPhases *)

    process CwPhases [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                     ( BusyList    : DecSet ,
                       WaitPairs,
                       HoldPairs   : SetOfPairs ,
                       CwList      : DecSet ,
                       CwPairs     : SetOfPairs ,
                       TwcList  ,
                       TwcUsers    :DecSet
                     )
    :noexit:=
    (
        CwLink  ?Called:DecDigit   !GetsCwTone  ?Caller:DecDigit
        [    (Called IsIn BusyList)
         and (Called ne Caller)
         and (Called IsIn CwList)
         and (FirstElement(Called,WaitPairs) eq none)
         and (SecondElement(Called,CwPairs) eq none)
	 and (Called NotIn TwcUsers)
        ] ;
        CwLink  !Called   !GetsCwTone ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( BusyList ,
                     InsertPairAtTheEnd (Pair (Called,Caller),
                      RemovePair (Pair (Caller,Caller),WaitPairs)),
                     InsertPairAtTheEnd (Pair (Called,Caller),HoldPairs),
                     CwList   ,
                     InsertPairAtTheEnd (Pair (Called,Caller),CwPairs)  ,
                     TwcList  ,
                     TwcUsers
                   )
   []   CwLink  ?X:DecDigit  !HangsUp  ?U:DecDigit
        [SecondElement (X,HoldPairs) eq U] ;
        PotsHang  !X   !HangsUp ;
        HangUpPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( X,
                     Remove ( X , BusyList ) ,
                     RemovePair (Pair(FirstElement(X,WaitPairs),X),WaitPairs),
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
   []   CwLink  ?X:DecDigit  !FlashHook ;
        (
           CwLink  !X  !FlashHook  ?U:DecDigit [U IsIn BusyList];
           CwLink  !U  ?Signal:Signal ;
           FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( X,U,
                     BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
        [] CwLink  !X  !FlashHook  !X  ?U:DecDigit [U NotIn BusyList] ;
           CwLink  !U  ?Signal:Signal ;
           FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   ( X,X,
                     BusyList ,
                     WaitPairs,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
        )
     )
     endproc (* CwPhases *)

    process HangUpPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                         (X           : DecDigit,     (* !X !HangsUp *)
                          BusyList    : DecSet ,
                          WaitPairs,
                          HoldPairs   : SetOfPairs ,
                          CwList      : DecSet ,
                          CwPairs     : SetOfPairs ,
                          TwcList  ,
                          TwcUsers    : DecSet 
                         )
    :noexit:=
    (  let T:DecDigit = SecondElement (X,CwPairs) in
       (
          [    (T eq none)
            or (T eq (SecondElement(X,WaitPairs)))
            or (T eq (SecondElement(X,HoldPairs)))
          ]  ->  GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                            (
                              BusyList ,
                              WaitPairs,
                              HoldPairs,
                              CwList   ,
                              CwPairs  ,
                              TwcList  ,
                              TwcUsers
                            )
         []
          [     (T ne none)
            and (T ne SecondElement(X,WaitPairs))
            and (T ne SecondElement(X,HoldPairs))
          ]  ->  GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                            (
                              BusyList ,
                              WaitPairs,
                              HoldPairs,
                              CwList   ,
                              RemovePair (Pair (X,T) , CwPairs ),
                              TwcList  ,
                              TwcUsers
                            )
       )
    )
    endproc (* HangUpPhase1 *)

    process FlashHookPhase1 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                          ( X,                (* !X !FlashHook !U *)
                            U           : DecDigit   ,
                            BusyList    : DecSet     ,
                            WaitPairs,
                            HoldPairs   : SetOfPairs ,
                            CwList      : DecSet     ,
                            CwPairs     : SetOfPairs ,
                            TwcList  ,
                            TwcUsers    : DecSet
                          )
    :noexit:=
    (  let T:DecDigit = SecondElement(X,CwPairs) in
       (
          [    (T eq SecondElement(X,HoldPairs))
            or (     (T IsIn BusyList)
                 and (SecondElement(T,HoldPairs) ne X)
               )
          ] -> FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                ( X,U,
                  BusyList,
                  RemovePair (Pair (X,SecondElement(X,WaitPairs)),WaitPairs),
                  RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                  CwList   ,
                  CwPairs  ,
                  TwcList  ,
                  TwcUsers
                )
       [] [     (T ne SecondElement(X,HoldPairs))
            and (    (T NotIn BusyList)
                  or (SecondElement(T,HoldPairs) eq X)
                )
          ] -> FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                ( X,U,
                  BusyList,
                  RemovePair (Pair (X,SecondElement(X,WaitPairs)),WaitPairs),
                  RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                  CwList   ,
                  RemovePair (Pair (X,T),CwPairs) ,
                  TwcList  ,
                  TwcUsers
                )
       )
    )
    endproc (* FlashHookPhase1*)

    process FlashHookPhase2 [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                          ( X,                (* !X !FlashHook !U *)
                            U           : DecDigit   ,
                            BusyList    : DecSet     ,
                            WaitPairs,
                            HoldPairs   : SetOfPairs ,
                            CwList      : DecSet     ,
                            CwPairs     : SetOfPairs ,
                            TwcList  ,
                            TwcUsers    : DecSet
                          )
    :noexit:=
    (
        [    (U NotIn BusyList)
          or (U eq X)
        ] ->  GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                        (
                          BusyList ,
                          WaitPairs,
                          HoldPairs,
                          CwList   ,
                          CwPairs  ,
                          TwcList  ,
                          TwcUsers
                        )
       []
        [     (U IsIn BusyList)
          and (U ne X)
        ] ->  GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                        (
                          BusyList ,
                          WaitPairs,
                          InsertPairAtTheEnd (Pair (X,U) , HoldPairs ),
                          CwList   ,
                          CwPairs  ,
                          TwcList  ,
                          TwcUsers
                        )
    )
    endproc (* FlashHookPhase2 *)


process TwcPhases [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     BusyList   : DecSet     ,
                     WaitPairs,
                     HoldPairs  : SetOfPairs ,
                     CwList     : DecSet     ,
                     CwPairs    : SetOfPairs ,
                     TwcList  ,
                     TwcUsers   : DecSet
                   )
:noexit:=
(
        TwcLink  ?user:DecDigit  !HangsUp
        [user IsIn TwcUsers] ;
        PotsHang !user  !HangsUp ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     Remove (user,BusyList),
                     RemovePair (Pair (FirstElement (user,WaitPairs),user),
                      WaitPairs ) ,
                     HoldPairs,
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     Remove (user,TwcUsers)
                   )
    []  TwcLink  ?X:DecDigit  !FlashHook  ?Y:DecDigit
        [     (X IsIn TwcList)
          and (Y NotIn TwcUsers )
        ] ;
        GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     BusyList ,
                     WaitPairs,
                     InsertPairAtTheEnd (Pair (X,Y) , HoldPairs),
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     Insert (X,TwcUsers)
                   )
    []  TwcLink  ?X:DecDigit  !FlashHook ;
        TwcLink  ?signal:Signal  !X ;
        (
           [    (signal IsEq TMustHangUp)
             or (signal IsEq TNotOnLine)
             or (signal IsEq YandTNotOnLine)
           ] ->
         GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     BusyList ,
                     WaitPairs,
                     RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     Remove (X,TwcUsers)
                   )
            []
            [     (signal IsNe TMustHangUp)
              and (signal IsNe TNotOnLine)
              and (signal IsNe YandTNotOnLine)
            ] ->
         GlobalView [PotsGate,PotsTalk,PotsHang,TwcLink,CwLink]
                   (
                     BusyList,
                     WaitPairs,
                     RemovePair (Pair (X,SecondElement(X,HoldPairs)),HoldPairs),
                     CwList   ,
                     CwPairs  ,
                     TwcList  ,
                     TwcUsers
                   )
        )

)
endproc (* TwcPhases *)

endproc (* GlobalView *)


process GlobalCwController [CwLink]
:noexit:=
(
    CwLink  ?X:DecDigit  !GetsCwTone  ?T:DecDigit ;
    CwLink  !X  !GetsCwTone ;
    (
        NewCwControllerTX [CwLink] (T,X)
        |||
        GlobalCwController [CwLink]
    )
)
endproc (* GlobalCwController *)

process NewCwControllerTX [CwLink] (T,X : DecDigit )
:noexit:=
(
    (
        CwLink  !X  !HangsUp  !T ;
        stop
    )
    []
    (
        CwLink  !X  !FlashHook ;
        (
            (
                CwLink  !X  !FlashHook  !X  ?Y:DecDigit [Y ne T] ;
                CwLink  !Y  !NotOnLine ;
                stop
            )
            []
            (
                CwLink  !X  !FlashHook  ?Y:DecDigit [Y ne X] ;
                CwLink  !Y  !OnLine ;
                CwControllerTX_TTalksToX [CwLink] (T,X,Y)
            )
        )
    )
)
endproc (* NewCwControllerTX *)

process CwControllerTX_TTalksToX [CwLink] (T,X,Y:DecDigit)
:noexit:=
(
    (
        CwLink  !X  !HangsUp  ?U:DecDigit [U ne T] ;
        stop
    )
    []
    (
        CwLink  !X  !FlashHook ;
        (
            (
                CwLink  !X  !FlashHook  !X  !T ;
                CwLink  !T  !NotOnLine ;
                stop
            )
            []
            (
                CwLink  !X  !FlashHook  !T ;
                CwLink  !T  !OnLine ;
                NewCwControllerTX [CwLink] (T,X)
            )
        )
    )
)
endproc (* CwControllerTX_TTalksToX *)

endspec
